@@ -1,7 +1,7 @@
 /*
  *      backendgvfs.c
  *
- *      Copyright 2008-2010 Enrico Tröger <enrico(at)xfce(dot)org>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -205,10 +205,6 @@ static void gigolo_backend_gvfs_class_init(GigoloBackendGVFSClass *klass)
 
 static void gigolo_backend_gvfs_finalize(GObject *object)
 {
-	GigoloBackendGVFS *self;
-
-	self = GIGOLO_BACKEND_GVFS(object);
-
 	G_OBJECT_CLASS(gigolo_backend_gvfs_parent_class)->finalize(object);
 }
 
@@ -222,6 +218,7 @@ static gchar *get_tooltip_text(GigoloBackendGVFS *backend, gpointer ref, gint re
 		case GIGOLO_WINDOW_REF_TYPE_MOUNT:
 		{
 			gchar *uri, *name, *clean_uri;
+			const gchar *bookmark_name = _("No bookmark");
 			GigoloBookmark *b;
 			GigoloSettings *settings;
 
@@ -235,10 +232,12 @@ static gchar *get_tooltip_text(GigoloBackendGVFS *backend, gpointer ref, gint re
 				const gchar *folder = gigolo_bookmark_get_folder(b);
 				if (NZV(folder))
 					setptr(clean_uri, g_build_filename(clean_uri, folder, NULL));
+				bookmark_name = gigolo_bookmark_get_name(b);
 			}
 
 			result = g_strdup_printf(
-				_("<b>%s</b>\n\nURI: %s\nConnected: Yes\nService Type: %s"), name, clean_uri, type);
+				_("<b>%s</b>\n\nURI: %s\nConnected: Yes\nService Type: %s\nBookmark: %s"),
+					name, clean_uri, type, bookmark_name);
 
 			g_free(clean_uri);
 			g_free(uri);
@@ -269,9 +268,11 @@ static void mount_volume_changed_cb(GVolumeMonitor *vm, G_GNUC_UNUSED GMount *mn
 	GVolume *volume;
 	GIcon *icon;
 	GtkTreeIter iter;
-	gchar *vol_name, *scheme, *uri, *tooltip_text;
+	gchar *vol_name, *mount_name, *display_name, *scheme, *uri, *tooltip_text;
 	const gchar *scheme_name;
 	GigoloBackendGVFSPrivate *priv = GIGOLO_BACKEND_GVFS_GET_PRIVATE(backend);
+	GigoloBookmark *bookmark;
+	GigoloSettings *settings = gigolo_window_get_settings(GIGOLO_WINDOW(priv->parent));
 
 	gtk_list_store_clear(priv->store);
 
@@ -279,13 +280,14 @@ static void mount_volume_changed_cb(GVolumeMonitor *vm, G_GNUC_UNUSED GMount *mn
 	mounts = g_volume_monitor_get_mounts(vm);
 	for (item = mounts; item != NULL; item = g_list_next(item))
 	{
+		display_name = NULL;
 		mount = G_MOUNT(item->data);
-		vol_name = g_mount_get_name(mount);
+		mount_name = g_mount_get_name(mount);
 		file = g_mount_get_root(mount);
 		scheme = g_file_get_uri_scheme(file);
 		if (gigolo_str_equal(scheme, "burn"))
 		{	/* ignore empty CDs which are listed as mounted to burn:// */
-			g_free(vol_name);
+			g_free(mount_name);
 			g_free(scheme);
 			g_object_unref(file);
 			continue;
@@ -295,9 +297,16 @@ static void mount_volume_changed_cb(GVolumeMonitor *vm, G_GNUC_UNUSED GMount *mn
 		icon = g_mount_get_icon(mount);
 		tooltip_text = get_tooltip_text(backend, mount, GIGOLO_WINDOW_REF_TYPE_MOUNT, scheme_name);
 
+		bookmark = gigolo_settings_get_bookmark_by_uri(settings, uri);
+		if (bookmark != NULL)
+		{
+			display_name = g_strdup_printf("%s (%s)",
+				gigolo_bookmark_get_name(bookmark), mount_name);
+		}
+
 		gtk_list_store_insert_with_values(priv->store, &iter, -1,
 				GIGOLO_WINDOW_COL_IS_MOUNTED, TRUE,
-				GIGOLO_WINDOW_COL_NAME, vol_name,
+				GIGOLO_WINDOW_COL_NAME, display_name ? display_name : mount_name,
 				GIGOLO_WINDOW_COL_SCHEME, scheme_name,
 				GIGOLO_WINDOW_COL_REF, mount,
 				GIGOLO_WINDOW_COL_REF_TYPE, GIGOLO_WINDOW_REF_TYPE_MOUNT,
@@ -305,7 +314,8 @@ static void mount_volume_changed_cb(GVolumeMonitor *vm, G_GNUC_UNUSED GMount *mn
 				GIGOLO_WINDOW_COL_ICON_NAME, "folder-remote",
 				GIGOLO_WINDOW_COL_TOOLTIP, tooltip_text,
 				-1);
-		g_free(vol_name);
+		g_free(mount_name);
+		g_free(display_name);
 		g_free(scheme);
 		g_free(uri);
 		g_free(tooltip_text);
@@ -430,26 +440,29 @@ static void volume_mount_finished_cb(GObject *src, GAsyncResult *res, gpointer b
 
 	if (! g_volume_mount_finish(G_VOLUME(src), res, &error))
 	{
-		gchar *name, *msg;
-
-		if (G_IS_VOLUME(src))
-			name = g_volume_get_name(G_VOLUME(src));
-		else
-		{
-			gigolo_backend_gvfs_get_name_and_uri_from_mount(G_MOUNT(src), &name, NULL);
-			if (name == NULL)
-				name = g_strdup(_("unknown"));
-		}
-
-		g_warning("Mounting of \"%s\" failed: %s", name, error->message);
 		if (! g_error_matches(error, G_IO_ERROR, G_IO_ERROR_FAILED_HANDLED))
 		{
-			msg = g_strdup_printf(_("Connecting to \"%s\" failed."), name);
-			g_signal_emit(backend, signals[OPERATION_FAILED], 0, msg, error->message);
-			g_free(msg);
+			gchar *name, *msg;
+
+			if (G_IS_VOLUME(src))
+				name = g_volume_get_name(G_VOLUME(src));
+			else
+			{
+				gigolo_backend_gvfs_get_name_and_uri_from_mount(G_MOUNT(src), &name, NULL);
+				if (name == NULL)
+					name = g_strdup(_("unknown"));
+			}
+
+			g_warning("Mounting of \"%s\" failed: %s", name, error->message);
+			if (! g_error_matches(error, G_IO_ERROR, G_IO_ERROR_FAILED_HANDLED))
+			{
+				msg = g_strdup_printf(_("Connecting to \"%s\" failed."), name);
+				g_signal_emit(backend, signals[OPERATION_FAILED], 0, msg, error->message);
+				g_free(msg);
+			}
+			g_error_free(error);
+			g_free(name);
 		}
-		g_error_free(error);
-		g_free(name);
 	}
 	else
 		verbose("Mount finished sucessfully");
@@ -466,25 +479,28 @@ static void unmount_finished_cb(GObject *src, GAsyncResult *res, gpointer backen
 	if (! g_mount_unmount_finish(G_MOUNT(src), res, &error))
 #endif
 	{
-		gchar *name, *msg;
-
-		if (G_IS_VOLUME(src))
-			name = g_volume_get_name(G_VOLUME(src));
-		else
+		if (! g_error_matches(error, G_IO_ERROR, G_IO_ERROR_FAILED_HANDLED))
 		{
-			gigolo_backend_gvfs_get_name_and_uri_from_mount(G_MOUNT(src), &name, NULL);
-			if (name == NULL)
-				name = g_strdup(_("unknown"));
+			gchar *name, *msg;
+
+			if (G_IS_VOLUME(src))
+				name = g_volume_get_name(G_VOLUME(src));
+			else
+			{
+				gigolo_backend_gvfs_get_name_and_uri_from_mount(G_MOUNT(src), &name, NULL);
+				if (name == NULL)
+					name = g_strdup(_("unknown"));
+			}
+
+			g_warning("Unmounting of \"%s\" failed: %s", name, error->message);
+			msg = g_strdup_printf(_("Disconnecting from \"%s\" failed."), name);
+
+			g_signal_emit(backend, signals[OPERATION_FAILED], 0, msg, error->message);
+
+			g_error_free(error);
+			g_free(name);
+			g_free(msg);
 		}
-
-		g_warning("Unmounting of \"%s\" failed: %s", name, error->message);
-		msg = g_strdup_printf(_("Disconnecting from \"%s\" failed."), name);
-
-		g_signal_emit(backend, signals[OPERATION_FAILED], 0, msg, error->message);
-
-		g_error_free(error);
-		g_free(name);
-		g_free(msg);
 	}
 }
 
@@ -529,14 +545,22 @@ void gigolo_backend_gvfs_unmount_mount(GigoloBackendGVFS *backend, gpointer moun
 }
 
 
+void gigolo_backend_gvfs_update_mounts_and_volumes(GigoloBackendGVFS *backend)
+{
+	GVolumeMonitor *gvm;
+
+	gvm = g_volume_monitor_get();
+	mount_volume_changed_cb(gvm, NULL, backend);
+}
+
+
 static void mount_ready_cb(GFile *location, GAsyncResult *res, MountInfo *mi)
 {
 	gchar *uri;
-	gboolean success;
 	GError *error = NULL;
 
 	uri = g_file_get_uri(location);
-	success = g_file_mount_enclosing_volume_finish(location, res, &error);
+	g_file_mount_enclosing_volume_finish(location, res, &error);
 
 	if (error != NULL && ! g_error_matches(error, G_IO_ERROR, G_IO_ERROR_ALREADY_MOUNTED))
 	{
@@ -592,6 +616,23 @@ gchar *gigolo_backend_gvfs_get_volume_identifier(gpointer volume)
 }
 
 
+gchar *gigolo_backend_gvfs_get_mount_path(gpointer mount)
+{
+	GFile *file;
+	gchar *path = NULL;
+
+	g_return_val_if_fail(mount != NULL, NULL);
+
+	file = g_mount_get_default_location(G_MOUNT(mount));
+	if (file != NULL)
+	{
+		path = g_file_get_path(file);
+		g_object_unref(file);
+	}
+	return path;
+}
+
+
 static gboolean browse_network_ready_cb(gpointer backend)
 {
 	GigoloBackendGVFSPrivate *priv;
@@ -611,14 +652,13 @@ static gboolean browse_network_ready_cb(gpointer backend)
 
 static void browse_network_mount_ready_cb(GFile *location, GAsyncResult *res, BrowseData *bd)
 {
-	gboolean success;
 	GError *error = NULL;
 	GigoloBackendGVFSPrivate *priv;
 
 	g_return_if_fail(bd != NULL);
 	g_return_if_fail(bd->self != NULL);
 
-	success = g_file_mount_enclosing_volume_finish(location, res, &error);
+	g_file_mount_enclosing_volume_finish(location, res, &error);
 
 	priv = GIGOLO_BACKEND_GVFS_GET_PRIVATE(bd->self);
 	priv->browse_counter--;
