@@ -1,7 +1,7 @@
 /*
  *      bookmark.c
  *
- *      Copyright 2008-2010 Enrico Tröger <enrico(at)xfce(dot)org>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@ struct _GigoloBookmarkPrivate
 	gchar	*share;
 	guint	 port;
 	gchar	*user;
+	gchar	*color;
 	gboolean autoconnect;
 	gboolean should_not_autoconnect;
 
@@ -54,7 +55,7 @@ static void gigolo_bookmark_finalize  		(GObject *object);
 G_DEFINE_TYPE(GigoloBookmark, gigolo_bookmark, G_TYPE_OBJECT);
 
 
-void gigolo_bookmark_bookmark_clear(GigoloBookmark *bookmark)
+void gigolo_bookmark_clear(GigoloBookmark *bookmark)
 {
 	GigoloBookmarkPrivate *priv = GIGOLO_BOOKMARK_GET_PRIVATE(bookmark);
 
@@ -95,7 +96,7 @@ static void gigolo_bookmark_class_init(GigoloBookmarkClass *klass)
 
 static void gigolo_bookmark_finalize(GObject *object)
 {
-	gigolo_bookmark_bookmark_clear(GIGOLO_BOOKMARK(object));
+	gigolo_bookmark_clear(GIGOLO_BOOKMARK(object));
 
 	G_OBJECT_CLASS(gigolo_bookmark_parent_class)->finalize(object);
 }
@@ -105,6 +106,7 @@ gboolean gigolo_bookmark_parse_uri(GigoloBookmark *bookmark, const gchar *uri)
 {
 	gchar *s, *t, *x, *end, *tmp;
 	guint l;
+	gboolean is_uri_dav;
 	GigoloBookmarkPrivate *priv = GIGOLO_BOOKMARK_GET_PRIVATE(bookmark);
 
 	priv->scheme = g_uri_parse_scheme(uri);
@@ -112,8 +114,8 @@ gboolean gigolo_bookmark_parse_uri(GigoloBookmark *bookmark, const gchar *uri)
 	s = strstr(uri, "://");
 	if (priv->scheme == NULL || s == NULL)
 	{
-		verbose("Error parsing URI '%s' while reading URI scheme", uri);
-		gigolo_bookmark_bookmark_clear(bookmark);
+		verbose("Error parsing URI '%s' at reading URI scheme", uri);
+		gigolo_bookmark_clear(bookmark);
 		return FALSE;
 	}
 	s += 3;
@@ -140,8 +142,8 @@ gboolean gigolo_bookmark_parse_uri(GigoloBookmark *bookmark, const gchar *uri)
 		}
 		if (l == 0)
 		{
-			verbose("Error parsing URI '%s' while reading username", uri);
-			gigolo_bookmark_bookmark_clear(bookmark);
+			verbose("Error parsing URI '%s' at reading username", uri);
+			gigolo_bookmark_clear(bookmark);
 			return FALSE;
 		}
 		tmp = g_strndup(s, l);
@@ -169,7 +171,7 @@ gboolean gigolo_bookmark_parse_uri(GigoloBookmark *bookmark, const gchar *uri)
 		if (! hostend || hostend > end)
 		{
 			verbose("Error parsing URI '%s', missing ']'", uri);
-			gigolo_bookmark_bookmark_clear(bookmark);
+			gigolo_bookmark_clear(bookmark);
 			return FALSE;
 		}
 		l = 0;
@@ -214,7 +216,6 @@ gboolean gigolo_bookmark_parse_uri(GigoloBookmark *bookmark, const gchar *uri)
 		priv->port = (guint) atoi(tmp);
 		g_free(tmp);
 	}
-	/* TODO handle WebDav path parts */
 	if (NZV(end) && *end == '/' && gigolo_str_equal("smb", priv->scheme))
 	{
 		end++; /* skip the slash */
@@ -228,6 +229,20 @@ gboolean gigolo_bookmark_parse_uri(GigoloBookmark *bookmark, const gchar *uri)
 		}
 		priv->share = g_strndup(end, l);
 	}
+	is_uri_dav = gigolo_str_equal("dav", priv->scheme) || gigolo_str_equal("davs", priv->scheme);
+	if (NZV(end) && *end == '/' && is_uri_dav)
+	{
+		end++; /* skip the slash */
+
+		/* remove trailing slashes */
+		l = strlen(end);
+		while (*end != '\0' && end[l-1] == '/')
+		{
+			l--;
+		}
+		if (l > 0)
+			priv->path = g_strndup(end, l);
+	}
 
 	return TRUE;
 }
@@ -235,7 +250,7 @@ gboolean gigolo_bookmark_parse_uri(GigoloBookmark *bookmark, const gchar *uri)
 
 static void gigolo_bookmark_init(GigoloBookmark *self)
 {
-	gigolo_bookmark_bookmark_clear(self);
+	gigolo_bookmark_clear(self);
 }
 
 
@@ -271,7 +286,7 @@ void gigolo_bookmark_clone(GigoloBookmark *dst, const GigoloBookmark *src)
 	priv_src = GIGOLO_BOOKMARK_GET_PRIVATE(src);
 
 	/* free existing strings and data */
-	gigolo_bookmark_bookmark_clear(dst);
+	gigolo_bookmark_clear(dst);
 
 	/* copy from src to dst */
 	priv_dst->name = g_strdup(priv_src->name);
@@ -309,7 +324,7 @@ static gchar *gigolo_bookmark_get_uri_real(GigoloBookmark *bookmark, gboolean es
 
 	domain = gigolo_bookmark_get_domain(bookmark);
 
-	result = g_strdup_printf("%s://%s%s%s%s%s%s/%s%s%s%s",
+	result = g_strdup_printf("%s://%s%s%s%s%s%s/%s%s%s",
 		priv->scheme,
 		(NZV(domain)) ? domain : "",
 		(NZV(domain)) ? ";" : "",
@@ -318,7 +333,6 @@ static gchar *gigolo_bookmark_get_uri_real(GigoloBookmark *bookmark, gboolean es
 		priv->host,
 		(port) ? port : "",
 		(NZV(priv->path)) ? priv->path : "",
-		(NZV(priv->path)) ? "/" : "",
 		(NZV(priv->share)) ? priv->share : "",
 		(NZV(priv->share)) ? "/" : "");
 
@@ -353,6 +367,8 @@ void gigolo_bookmark_set_uri(GigoloBookmark *bookmark, const gchar *uri)
 	tmp = gigolo_bookmark_new_from_uri(priv->name, uri);
 	if (gigolo_bookmark_is_valid(tmp))
 		gigolo_bookmark_clone(bookmark, tmp);
+
+	g_object_unref(tmp);
 }
 
 
@@ -440,8 +456,12 @@ gchar *gigolo_bookmark_get_folder_expanded(GigoloBookmark *bookmark)
 
 	folder = GIGOLO_BOOKMARK_GET_PRIVATE(bookmark)->folder;
 	username = gigolo_bookmark_get_user(bookmark);
-	if (NZV(folder) && username && folder[0] == '~')
+	if (NZV(folder) && folder[0] == '~')
+	{
+		if (! username)
+			username = g_get_user_name();
 		result = g_strconcat("/home/", username, folder + 1, NULL);
+	}
 	else
 		result = g_strdup(folder);
 
@@ -629,6 +649,28 @@ void gigolo_bookmark_set_domain(GigoloBookmark *bookmark, const gchar *domain)
 }
 
 
+const gchar *gigolo_bookmark_get_color(GigoloBookmark *bookmark)
+{
+	g_return_val_if_fail(bookmark != NULL, NULL);
+
+	return GIGOLO_BOOKMARK_GET_PRIVATE(bookmark)->color;
+}
+
+
+void gigolo_bookmark_set_color(GigoloBookmark *bookmark, const gchar *color)
+{
+	GigoloBookmarkPrivate *priv;
+
+	g_return_if_fail(bookmark != NULL);
+	g_return_if_fail(color != NULL);
+
+	priv = GIGOLO_BOOKMARK_GET_PRIVATE(bookmark);
+
+	g_free(priv->color);
+	priv->color = g_strdup(color);
+}
+
+
 gboolean gigolo_bookmark_is_valid(GigoloBookmark *bookmark)
 {
 	GigoloBookmarkPrivate *priv;
