@@ -1,7 +1,7 @@
 /*
  *      bookmark.h
  *
- *      Copyright 2008-2009 Enrico Tröger <enrico(at)xfce(dot)org>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -65,6 +65,13 @@ void				gigolo_bookmark_set_scheme		(GigoloBookmark *bookmark, const gchar *sche
 const gchar*		gigolo_bookmark_get_host		(GigoloBookmark *bookmark);
 void				gigolo_bookmark_set_host		(GigoloBookmark *bookmark, const gchar *host);
 
+const gchar*		gigolo_bookmark_get_folder		(GigoloBookmark *bookmark);
+void				gigolo_bookmark_set_folder		(GigoloBookmark *bookmark, const gchar *folder);
+gchar*				gigolo_bookmark_get_folder_expanded(GigoloBookmark *bookmark);
+
+const gchar*		gigolo_bookmark_get_path		(GigoloBookmark *bookmark);
+void				gigolo_bookmark_set_path		(GigoloBookmark *bookmark, const gchar *path);
+
 guint				gigolo_bookmark_get_port		(GigoloBookmark *bookmark);
 void				gigolo_bookmark_set_port		(GigoloBookmark *bookmark, guint port);
 
@@ -84,6 +91,12 @@ void				gigolo_bookmark_set_autoconnect	(GigoloBookmark *bookmark, gboolean auto
 gboolean			gigolo_bookmark_get_should_not_autoconnect	(GigoloBookmark *bookmark);
 void				gigolo_bookmark_set_should_not_autoconnect	(GigoloBookmark *bookmark, gboolean should_not_autoconnect);
 
+gboolean			gigolo_bookmark_parse_uri		(GigoloBookmark *bookmark, const gchar *uri);
+
+void				gigolo_bookmark_clear			(GigoloBookmark *bookmark);
+
+const gchar*		gigolo_bookmark_get_color		(GigoloBookmark *bookmark);
+void				gigolo_bookmark_set_color		(GigoloBookmark *bookmark, const gchar *color);
 
 G_END_DECLS
 
