@@ -1,7 +1,7 @@
 /*
  *      bookmarkeditdialog.c
  *
- *      Copyright 2008-2010 Enrico Tröger <enrico(at)xfce(dot)org>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -50,6 +50,8 @@ struct _GigoloBookmarkEditDialogPrivate
 	GtkWidget *type_combo;
 	GtkWidget *information_label;
 
+	GtkWidget *separator;
+
 	GtkWidget *name_label;
 	GtkWidget *name_entry;
 
@@ -82,6 +84,10 @@ struct _GigoloBookmarkEditDialogPrivate
 	GtkWidget *share_button;
 	GtkWidget *share_entry;
 
+	GtkWidget *color_label;
+	GtkWidget *color_chooser;
+	gboolean   color_set;
+
 	GigoloBookmark *bookmark_init;
 	GigoloBookmark *bookmark_update;
 
@@ -188,6 +194,19 @@ static void gigolo_bookmark_edit_dialog_destroy(GtkObject *object)
 }
 
 
+static gboolean check_custom_uri(const gchar *uri)
+{
+	GigoloBookmark *bm;
+	gboolean result;
+
+	bm = gigolo_bookmark_new_from_uri("(validation)", uri);
+	result = gigolo_bookmark_is_valid(bm);
+	g_object_unref(bm);
+
+	return result;
+}
+
+
 gint gigolo_bookmark_edit_dialog_run(GigoloBookmarkEditDialog *dialog)
 {
 	gint res;
@@ -262,7 +281,7 @@ gint gigolo_bookmark_edit_dialog_run(GigoloBookmarkEditDialog *dialog)
 			if (! error && gtk_widget_get_parent(priv->uri_entry) != NULL)
 			{
 				tmp = gtk_entry_get_text(GTK_ENTRY(priv->uri_entry));
-				if (! *tmp)
+				if (! *tmp || ! check_custom_uri(tmp))
 				{
 					error = TRUE;
 					gigolo_message_dialog(dialog, GTK_MESSAGE_ERROR, _("Error"),
@@ -275,7 +294,9 @@ gint gigolo_bookmark_edit_dialog_run(GigoloBookmarkEditDialog *dialog)
 				tmp = gtk_entry_get_text(GTK_ENTRY(priv->path_entry));
 				if (tmp[0] == '/')
 				{	/* remove leading slashes */
-					gtk_entry_set_text(GTK_ENTRY(priv->path_entry), tmp + 1);
+					gchar *path = g_strdup(tmp);
+					gtk_entry_set_text(GTK_ENTRY(priv->path_entry), path + 1);
+					g_free(path);
 				}
 			}
 			if (! error && gtk_widget_get_parent(priv->share_entry) != NULL)
@@ -283,7 +304,9 @@ gint gigolo_bookmark_edit_dialog_run(GigoloBookmarkEditDialog *dialog)
 				tmp = gtk_entry_get_text(GTK_ENTRY(priv->share_entry));
 				if (tmp[0] == '/')
 				{	/* remove leading slashes */
-					gtk_entry_set_text(GTK_ENTRY(priv->share_entry), tmp + 1);
+					gchar *share = g_strdup(tmp);
+					gtk_entry_set_text(GTK_ENTRY(priv->share_entry), share + 1);
+					g_free(share);
 				}
 			}
 			if (! error)
@@ -402,6 +425,14 @@ static void init_values(GigoloBookmarkEditDialog *dialog)
 	tmp = gigolo_bookmark_get_name(priv->bookmark_init);
 	if (tmp != NULL)
 		gtk_entry_set_text(GTK_ENTRY(priv->name_entry), tmp);
+	/* Color */
+	tmp = gigolo_bookmark_get_color(priv->bookmark_init);
+	if (tmp != NULL)
+	{
+		GdkColor color;
+		if (gdk_color_parse(tmp, &color))
+			gtk_color_button_set_color(GTK_COLOR_BUTTON(priv->color_chooser), &color);
+	}
 	/* URI */
 	uri = gigolo_bookmark_get_uri(priv->bookmark_init);
 	if (uri != NULL)
@@ -530,7 +561,7 @@ static void setup_for_type(GigoloBookmarkEditDialog *dialog)
 		gtk_container_remove(GTK_CONTAINER(priv->table), priv->information_label);
 	}
 
-	i = 4;
+	i = 6;
 	table = priv->table;
 
 	if (meth->scheme == NULL)
@@ -740,6 +771,25 @@ static void fill_method_combo_box(GigoloBookmarkEditDialog *dialog)
 }
 
 
+static void update_bookmark_color(GigoloBookmarkEditDialog *dialog)
+{
+	GigoloBookmarkEditDialogPrivate *priv;
+	GdkColor color;
+	gchar *color_string;
+
+	priv = GIGOLO_BOOKMARK_EDIT_DIALOG_GET_PRIVATE(dialog);
+
+	if (! priv->color_set)
+		/* if no colour has been chosen by the user, don't set the default colour (black) */
+		return;
+
+	gtk_color_button_get_color(GTK_COLOR_BUTTON(priv->color_chooser), &color);
+	color_string = gdk_color_to_string(&color);
+	gigolo_bookmark_set_color(priv->bookmark_update, color_string);
+	g_free(color_string);
+}
+
+
 /* Update the contents of the bookmark with the values from the dialog. */
 static void update_bookmark(GigoloBookmarkEditDialog *dialog)
 {
@@ -767,6 +817,8 @@ static void update_bookmark(GigoloBookmarkEditDialog *dialog)
 	else
 		gigolo_bookmark_set_name(priv->bookmark_update, GIGOLO_BOOKMARK_NAME_NONE);
 
+	update_bookmark_color(dialog);
+
 	if (idx == -1)
 		idx = 0;
 	if (methods[idx].scheme == NULL)
@@ -862,8 +914,11 @@ static void gigolo_bookmark_edit_dialog_set_property(GObject *object, guint prop
 				combo_set_active(priv->type_combo, 0);
 				gtk_widget_hide(priv->name_label);
 				gtk_widget_hide(priv->name_entry);
+				gtk_widget_hide(priv->color_label);
+				gtk_widget_hide(priv->color_chooser);
 				gtk_widget_hide(priv->autoconnect_label);
 				gtk_widget_hide(priv->autoconnect_checkbtn);
+				gtk_widget_hide(priv->separator);
 				break;
 			}
 		}
@@ -894,6 +949,8 @@ static void browse_host_finished_cb(G_GNUC_UNUSED GigoloBackendGVFS *bnd, GSList
 		{
 			gtk_combo_box_append_text(GTK_COMBO_BOX(priv->share_combo), node->data);
 		}
+		gtk_combo_box_set_active(GTK_COMBO_BOX(priv->share_combo), 0);
+
 	}
 	gtk_widget_set_sensitive(priv->share_button, TRUE);
 }
@@ -931,6 +988,15 @@ static void entry_activate_cb(G_GNUC_UNUSED GtkEditable *editable, GigoloBookmar
 }
 
 
+static void color_chooser_set_cb(G_GNUC_UNUSED GtkColorButton *widget,
+								 GigoloBookmarkEditDialog *dialog)
+{
+	GigoloBookmarkEditDialogPrivate *priv = GIGOLO_BOOKMARK_EDIT_DIALOG_GET_PRIVATE(dialog);
+
+	priv->color_set = TRUE;
+}
+
+
 static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 {
 	GtkWidget *label;
@@ -975,23 +1041,37 @@ static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 	gtk_label_set_mnemonic_widget(GTK_LABEL(priv->name_label), entry);
 	gtk_table_attach(GTK_TABLE(table), entry, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
 
+	priv->color_label = gtk_label_new_with_mnemonic(_("_Color:"));
+	gtk_misc_set_alignment(GTK_MISC(priv->color_label), 0.0, 0.5);
+	gtk_table_attach(GTK_TABLE(table), priv->color_label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+
+	priv->color_set = FALSE;
+	priv->color_chooser = gtk_color_button_new();
+	g_signal_connect(priv->color_chooser, "color-set", G_CALLBACK(color_chooser_set_cb), dialog);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(priv->color_label), priv->color_chooser);
+	gtk_table_attach(GTK_TABLE(table), priv->color_chooser,
+		1, 2, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+
 	priv->autoconnect_label = gtk_label_new_with_mnemonic(_("Au_to-Connect"));
 	gtk_misc_set_alignment(GTK_MISC(priv->autoconnect_label), 0.0, 0.5);
-	gtk_table_attach(GTK_TABLE(table), priv->autoconnect_label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), priv->autoconnect_label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 
 	priv->autoconnect_checkbtn = gtk_check_button_new();
 	gtk_label_set_mnemonic_widget(GTK_LABEL(priv->autoconnect_label), priv->autoconnect_checkbtn);
-	gtk_table_attach(GTK_TABLE(table), priv->autoconnect_checkbtn, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), priv->autoconnect_checkbtn, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
+
+	priv->separator = gtk_hseparator_new();
+	gtk_table_attach(GTK_TABLE(table), priv->separator, 0, 2, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
 
 	label = gtk_label_new_with_mnemonic(_("Service t_ype:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
 
 	priv->type_combo = combo = gtk_combo_box_new();
-	gtk_table_attach(GTK_TABLE(table), combo, 1, 2, 2, 3, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), combo, 1, 2, 4, 5, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
 
 	label_tmp = gtk_label_new(" ");
-	gtk_table_attach(GTK_TABLE(table), label_tmp, 0, 2, 3, 4, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), label_tmp, 0, 2, 5, 6, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
 
 	renderer = gtk_cell_renderer_text_new();
 	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), renderer, TRUE);
@@ -1009,6 +1089,8 @@ static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 	priv->port_spin = gtk_spin_button_new_with_range(0, 65535, 1);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(priv->port_spin), 0.0);
 	gtk_widget_set_tooltip_text(priv->port_spin, _("Set the port to 0 to use the default port"));
+	gtk_widget_set_tooltip_text(priv->folder_entry,
+		_("This is not used for the actual mount, only necessary for opening the mount point in a file browser"));
 	priv->user_entry = gtk_entry_new();
 	priv->domain_entry = gtk_entry_new();
 	priv->share_combo = gtk_combo_box_entry_new_text();
@@ -1017,7 +1099,7 @@ static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 	priv->uri_label = gtk_label_new_with_mnemonic(_("_Location (URI):"));
 	priv->host_label = gtk_label_new_with_mnemonic(_("_Server:"));
 	priv->folder_label = gtk_label_new_with_mnemonic(_("_Folder:"));
-	priv->path_label = gtk_label_new_with_mnemonic(_("_Path:"));
+	priv->path_label = gtk_label_new_with_mnemonic(_("P_ath:"));
 	priv->user_label = gtk_label_new_with_mnemonic(_("_User Name:"));
 	priv->information_label = gtk_label_new(_("Optional information:"));
 	priv->port_label = gtk_label_new_with_mnemonic(_("_Port:"));
