@@ -1,7 +1,7 @@
 /*
  *      main.c
  *
- *      Copyright 2008-2010 Enrico Tröger <enrico(at)xfce(dot)org>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -39,10 +39,12 @@
 static gboolean show_version = FALSE;
 static gboolean list_schemes = FALSE;
 static gboolean new_instance = FALSE;
+static gboolean auto_connect = FALSE;
 extern gboolean verbose_mode;
 
 static GOptionEntry cli_options[] =
 {
+	{ "auto-connect", 'a', 0, G_OPTION_ARG_NONE, &auto_connect, N_("Connect all bookmarks marked as 'auto connect' and exit"), NULL },
 	{ "new-instance", 'i', 0, G_OPTION_ARG_NONE, &new_instance, N_("Ignore running instances, enforce opening a new instance"), NULL },
 	{ "list-schemes", 'l', 0, G_OPTION_ARG_NONE, &list_schemes, N_("Print a list of supported URI schemes"), NULL },
 	{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose_mode, N_("Be verbose"), NULL },
@@ -51,6 +53,37 @@ static GOptionEntry cli_options[] =
 };
 
 
+static gboolean auto_connect_bookmarks(void)
+{
+	GigoloBackendGVFS *backend_gvfs;
+	GigoloSettings *settings;
+	GigoloBookmarkList *bookmarks;
+	GigoloBookmark *bm;
+	guint i;
+	gchar *uri;
+
+	backend_gvfs = gigolo_backend_gvfs_new();
+	settings = gigolo_settings_new();
+	bookmarks = gigolo_settings_get_bookmarks(settings);
+
+	for (i = 0; i < bookmarks->len; i++)
+	{
+		bm = g_ptr_array_index(bookmarks, i);
+		if (gigolo_bookmark_get_autoconnect(bm) && ! gigolo_bookmark_get_should_not_autoconnect(bm))
+		{
+			uri = gigolo_bookmark_get_uri_escaped(bm);
+			/* Mounting happens asynchronously here and so we don't wait until it is finished
+			 * nor de we get any feedback or errors.
+			 * TODO make this synchronous(looping and checking) and check for errors */
+			gigolo_backend_gvfs_mount_uri(backend_gvfs, uri, NULL, NULL, FALSE);
+			g_free(uri);
+		}
+	}
+
+	return TRUE;
+}
+
+
 static void print_supported_schemes(void)
 {
 	const gchar* const *supported;
@@ -88,7 +121,7 @@ gint main(gint argc, gchar** argv)
 	if (show_version)
 	{
 		g_print("%s %s\n\n", PACKAGE, VERSION);
-		g_print("%s\n", "Copyright (c) 2008-2010");
+		g_print("%s\n", "Copyright (c) 2008-2011");
 		g_print("\tEnrico Tröger <enrico@xfce.org>\n\n");
 		g_print("\n");
 
@@ -102,6 +135,13 @@ gint main(gint argc, gchar** argv)
 		return EXIT_SUCCESS;
 	}
 
+	if (auto_connect)
+	{
+		gboolean ret = auto_connect_bookmarks();
+
+		return ret ? EXIT_SUCCESS : EXIT_FAILURE;
+	}
+
 	if (! new_instance)
 	{
 		gis = gigolo_single_instance_new();
