@@ -1,7 +1,7 @@
 /*
  *      settings.c
  *
- *      Copyright 2008-2010 Enrico Tröger <enrico(at)xfce(dot)org>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -52,9 +52,11 @@ struct _GigoloSettingsPrivate
 	gint		view_mode;
 	gboolean	show_panel;
 	guint		last_panel_page;
+	gint		panel_position;
 	gboolean	show_autoconnect_errors;
 
 	gchar		*file_manager;
+	gchar		*terminal;
 	gint		 autoconnect_interval;
 	gint		*geometry; /* window size and position, field 4 is a flag for maximized state */
 
@@ -76,6 +78,7 @@ enum
 	PROP_0,
 
 	PROP_FILE_MANAGER,
+	PROP_TERMINAL,
 	PROP_AUTOCONNECT_INTERVAL,
 
 	PROP_SAVE_GEOMETRY,
@@ -87,6 +90,7 @@ enum
 	PROP_VIEW_MODE,
 	PROP_SHOW_PANEL,
 	PROP_LAST_PANEL_PAGE,
+	PROP_PANEL_POSITION,
 	PROP_SHOW_AUTOCONNECT_ERRORS
 };
 
@@ -125,6 +129,10 @@ static void gigolo_settings_set_property(GObject *object, guint prop_id, const G
 		g_free(priv->file_manager);
 		priv->file_manager = g_value_dup_string(value);
 		break;
+	case PROP_TERMINAL:
+		g_free(priv->terminal);
+		priv->terminal = g_value_dup_string(value);
+		break;
 	case PROP_AUTOCONNECT_INTERVAL:
 		priv->autoconnect_interval = g_value_get_int(value);
 		break;
@@ -134,6 +142,9 @@ static void gigolo_settings_set_property(GObject *object, guint prop_id, const G
 	case PROP_LAST_PANEL_PAGE:
 		priv->last_panel_page = g_value_get_uint(value);
 		break;
+	case PROP_PANEL_POSITION:
+		priv->panel_position = g_value_get_uint(value);
+		break;
 	case PROP_SHOW_AUTOCONNECT_ERRORS:
 		priv->show_autoconnect_errors = g_value_get_boolean(value);
 		break;
@@ -174,6 +185,9 @@ static void gigolo_settings_get_property(GObject *object, guint prop_id, GValue
 	case PROP_FILE_MANAGER:
 		g_value_set_string(value, priv->file_manager);
 		break;
+	case PROP_TERMINAL:
+		g_value_set_string(value, priv->terminal);
+		break;
 	case PROP_AUTOCONNECT_INTERVAL:
 		if (priv->autoconnect_interval < 0)
 			g_object_set(object, "autoconnect-interval", DEFAULT_AUTOCONNECT_INTERVAL, NULL);
@@ -185,6 +199,9 @@ static void gigolo_settings_get_property(GObject *object, guint prop_id, GValue
 	case PROP_LAST_PANEL_PAGE:
 		g_value_set_uint(value, priv->last_panel_page);
 		break;
+	case PROP_PANEL_POSITION:
+		g_value_set_uint(value, priv->panel_position);
+		break;
 	case PROP_SHOW_AUTOCONNECT_ERRORS:
 		g_value_set_boolean(value, priv->show_autoconnect_errors);
 		break;
@@ -265,7 +282,15 @@ static void gigolo_settings_class_init(GigoloSettingsClass *klass)
 									g_param_spec_string(
 									"file-manager",
 									"file-manager",
-									"A program for use to open mount points",
+									"A program to use to open mount points",
+									NULL,
+									G_PARAM_READWRITE));
+	g_object_class_install_property(gobject_class,
+									PROP_TERMINAL,
+									g_param_spec_string(
+									"terminal",
+									"terminal",
+									"A program to use to open mount points in a terminal",
 									NULL,
 									G_PARAM_READWRITE));
 	g_object_class_install_property(gobject_class,
@@ -292,6 +317,14 @@ static void gigolo_settings_class_init(GigoloSettingsClass *klass)
 									"Last displayed panel page",
 									0, G_MAXUINT, 0,
 									G_PARAM_READWRITE));
+	g_object_class_install_property(gobject_class,
+									PROP_PANEL_POSITION,
+									g_param_spec_uint(
+									"panel-position",
+									"panel-position",
+									"Last panel position",
+									0, G_MAXUINT, 0,
+									G_PARAM_READWRITE));
 	g_object_class_install_property(gobject_class,
 									PROP_SHOW_AUTOCONNECT_ERRORS,
 									g_param_spec_boolean(
@@ -380,6 +413,7 @@ static void write_data(GKeyFile *k, const gchar *filename)
 	gsize len;
 	GError *error = NULL;
 	gchar *data;
+	gchar *real_filename;
 
 	data = g_key_file_to_data(k, &len, &error);
 	if (data == NULL || error != NULL)
@@ -389,13 +423,31 @@ static void write_data(GKeyFile *k, const gchar *filename)
 		g_free(data);
 		return;
 	}
+	/* resolve symbolic links */
+	if (g_file_test(filename, G_FILE_TEST_IS_SYMLINK))
+	{
+		real_filename = g_file_read_link(filename, &error);
+		if (error)
+		{
+			g_warning("Writing configuration file to disk failed (%s).", error->message);
+			g_error_free(error);
+			g_free(data);
+			return;
+		}
+	}
+	else
+	{
+		real_filename = g_strdup(filename);
+	}
 
-	if (! g_file_set_contents(filename, data, len, &error))
+	/* write data to file */
+	if (! g_file_set_contents(real_filename, data, len, &error))
 	{
 		g_warning("Writing configuration file to disk failed (%s).", error->message);
 		g_error_free(error);
 	}
 	g_free(data);
+	g_free(real_filename);
 }
 
 
@@ -411,6 +463,8 @@ static void write_settings_config(GigoloSettings *settings)
 
 	if (priv->file_manager != NULL)
 		g_key_file_set_string(k, SECTION_GENERAL, "file_manager", priv->file_manager);
+	if (priv->terminal != NULL)
+		g_key_file_set_string(k, SECTION_GENERAL, "terminal", priv->terminal);
 	g_key_file_set_integer(k, SECTION_GENERAL, "autoconnect_interval", priv->autoconnect_interval);
 
 	if (priv->geometry != NULL)
@@ -424,6 +478,7 @@ static void write_settings_config(GigoloSettings *settings)
 	g_key_file_set_integer(k, SECTION_UI, "view_mode", priv->view_mode);
 	g_key_file_set_boolean(k, SECTION_UI, "show_panel", priv->show_panel);
 	g_key_file_set_integer(k, SECTION_UI, "last_panel_page", priv->last_panel_page);
+	g_key_file_set_integer(k, SECTION_UI, "panel_position", priv->panel_position);
 	g_key_file_set_boolean(k, SECTION_UI, "show_autoconnect_errors", priv->show_autoconnect_errors);
 
 	write_data(k, priv->config_filename);
@@ -460,6 +515,7 @@ static void write_settings_bookmarks(GigoloSettings *settings)
 			set_setting_string(k, name, "share", gigolo_bookmark_get_share(bm));
 			set_setting_string(k, name, "domain", gigolo_bookmark_get_domain(bm));
 			set_setting_string(k, name, "path", gigolo_bookmark_get_path(bm));
+			set_setting_string(k, name, "color", gigolo_bookmark_get_color(bm));
 			set_setting_int(k, name, "port", gigolo_bookmark_get_port(bm));
 			set_setting_int(k, name, "autoconnect", gigolo_bookmark_get_autoconnect(bm));
 		}
@@ -517,6 +573,8 @@ static void load_settings_read_config(GigoloSettingsPrivate *priv)
 	}
 
 	priv->file_manager = get_setting_string(k, SECTION_GENERAL, "file_manager", "gvfs-open");
+	/* is there a generic way to open a default terminal? */
+	priv->terminal = get_setting_string(k, SECTION_GENERAL, "terminal", "xterm");
 	priv->autoconnect_interval = get_setting_int(k, SECTION_GENERAL, "autoconnect_interval",
 		DEFAULT_AUTOCONNECT_INTERVAL);
 
@@ -530,6 +588,7 @@ static void load_settings_read_config(GigoloSettingsPrivate *priv)
 	priv->toolbar_style = get_setting_int(k, SECTION_UI, "toolbar_style", -1);
 	priv->toolbar_orientation = get_setting_int(k, SECTION_UI, "toolbar_orientation", 0);
 	priv->view_mode = get_setting_int(k, SECTION_UI, "view_mode", 0);
+	priv->panel_position = get_setting_int(k, SECTION_UI, "panel_position", 0);
 	priv->geometry = g_key_file_get_integer_list(k, SECTION_UI, "geometry", NULL, &error);
 	if (error)
 	{
@@ -561,7 +620,7 @@ static void load_settings_read_bookmarks(GigoloSettingsPrivate *priv)
 	GError *error = NULL;
 	gsize len, i;
 	gchar **groups;
-	gchar *scheme, *host, *user, *domain, *share, *folder, *path;
+	gchar *scheme, *host, *user, *domain, *share, *folder, *path, *color;
 	gint port;
 	gboolean autoconnect;
 	GigoloBookmark *bm;
@@ -585,6 +644,7 @@ static void load_settings_read_bookmarks(GigoloSettingsPrivate *priv)
 		user = get_setting_string(k, groups[i], "user", "");
 		domain = get_setting_string(k, groups[i], "domain", "");
 		share = get_setting_string(k, groups[i], "share", "");
+		color = get_setting_string(k, groups[i], "color", "");
 		port = get_setting_int(k, groups[i], "port", 0);
 		autoconnect = get_setting_int(k, groups[i], "autoconnect", FALSE);
 
@@ -603,6 +663,8 @@ static void load_settings_read_bookmarks(GigoloSettingsPrivate *priv)
 			gigolo_bookmark_set_path(bm, path);
 		if (NZV(share))
 			gigolo_bookmark_set_share(bm, share);
+		if (NZV(color))
+			gigolo_bookmark_set_color(bm, color);
 		gigolo_bookmark_set_port(bm, port);
 		gigolo_bookmark_set_autoconnect(bm, autoconnect);
 
@@ -615,6 +677,7 @@ static void load_settings_read_bookmarks(GigoloSettingsPrivate *priv)
 		g_free(user);
 		g_free(domain);
 		g_free(share);
+		g_free(color);
 	}
 	g_strfreev(groups);
 
@@ -758,6 +821,18 @@ gboolean gigolo_settings_has_file_manager(GigoloSettings *settings)
 }
 
 
+gboolean gigolo_settings_has_terminal(GigoloSettings *settings)
+{
+	GigoloSettingsPrivate *priv;
+
+	g_return_val_if_fail(settings != NULL, FALSE);
+
+	priv = GIGOLO_SETTINGS_GET_PRIVATE(settings);
+
+	return NZV(priv->terminal);
+}
+
+
 GigoloBookmark *gigolo_settings_get_bookmark_by_uri(GigoloSettings *settings, const gchar *uri)
 {
 	GigoloBookmarkList *bml;
