@@ -1,7 +1,7 @@
 /*
  *      window.c
  *
- *      Copyright 2008-2009 Enrico Tröger <enrico(at)xfce(dot)org>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(at)xfce(dot)org>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -30,14 +30,15 @@
 #include "common.h"
 #include "bookmark.h"
 #include "settings.h"
+#include "backendgvfs.h"
 #include "window.h"
 #include "bookmarkdialog.h"
 #include "bookmarkeditdialog.h"
 #include "menubuttonaction.h"
 #include "preferencesdialog.h"
-#include "backendgvfs.h"
 #include "mountdialog.h"
 #include "browsenetworkpanel.h"
+#include "bookmarkpanel.h"
 
 
 typedef struct _GigoloWindowPrivate			GigoloWindowPrivate;
@@ -58,7 +59,10 @@ struct _GigoloWindowPrivate
 	GtkWidget		*hbox_pane;
 	GtkWidget		*hbox_view;
 
+	GtkWidget		*panel_pane;
 	GtkWidget		*browse_panel;
+	GtkWidget		*bookmark_panel;
+	GtkWidget		*notebook_panel;
 	GtkWidget		*treeview;
 	GtkWidget		*iconview;
 	GtkWidget		*swin_treeview;
@@ -70,6 +74,7 @@ struct _GigoloWindowPrivate
 	GtkAction		*action_bookmarks;
 	GtkAction		*action_bookmark_create;
 	GtkAction		*action_open;
+	GtkAction		*action_open_terminal;
 	GtkAction		*action_copyuri;
 
 	GtkActionGroup	*action_group;
@@ -120,25 +125,32 @@ static void gigolo_window_destroy(GigoloWindow *window)
 
 	if (gigolo_settings_get_boolean(priv->settings, "save-geometry"))
 	{
+		GdkWindow *gdk_window = gigolo_widget_get_window(GTK_WIDGET(window));
 		gtk_window_get_position(GTK_WINDOW(window), &geo[0], &geo[1]);
 		gtk_window_get_size(GTK_WINDOW(window), &geo[2], &geo[3]);
-		if (gdk_window_get_state(
-				gigolo_widget_get_window(GTK_WIDGET(window))) & GDK_WINDOW_STATE_MAXIMIZED)
+		if (gdk_window != NULL && gdk_window_get_state(gdk_window) & GDK_WINDOW_STATE_MAXIMIZED)
 			geo[4] = 1;
 		else
 			geo[4] = 0;
 
 		gigolo_settings_set_geometry(priv->settings, geo, 5);
+
+		g_object_set(priv->settings, "panel-position",
+			gtk_paned_get_position(GTK_PANED(priv->panel_pane)), NULL);
 	}
+	g_object_set(priv->settings, "last-panel-page",
+		gtk_notebook_get_current_page(GTK_NOTEBOOK(priv->notebook_panel)), NULL);
+
 	gtk_widget_destroy(priv->tree_popup_menu);
 	gtk_widget_destroy(priv->systray_icon_popup_menu);
 	gtk_widget_destroy(priv->swin_treeview);
 	gtk_widget_destroy(priv->swin_iconview);
-	gtk_widget_destroy(priv->toolbar);
+	g_object_unref(priv->toolbar);
 	g_object_unref(priv->action_group);
 	g_object_unref(priv->systray_icon);
 	g_object_unref(priv->systray_icon_popup_menu);
 	g_object_unref(priv->backend_gvfs);
+	priv->backend_gvfs = NULL;
 
 	gtk_widget_destroy(GTK_WIDGET(window));
 
@@ -211,50 +223,21 @@ static void get_selected_iter(GigoloWindow *window, GtkTreeIter *iter)
 	}
 	else
 	{
-		GList *l, *items = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(priv->iconview));
+		GList *items = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(priv->iconview));
 		GtkTreeModel *model = gtk_icon_view_get_model(GTK_ICON_VIEW(priv->iconview));
 
-		for (l = items; l != NULL; l = l->next)
+		if (items != NULL)
 		{
-			gtk_tree_model_get_iter(model, iter, l->data);
-			/* the selection mode is SINGLE, so the list should never have more than one entry */
-			break;
+			/* the selection mode is SINGLE, so the list should never have more than one entry
+			 * and we simply choose the first item */
+			gtk_tree_model_get_iter(model, iter, items->data);
 		}
-
 		g_list_foreach(items, (GFunc) gtk_tree_path_free, NULL);
 		g_list_free(items);
 	}
 }
 
 
-GigoloBookmark *gigolo_window_find_bookmark_by_uri(GigoloWindow *window, const gchar *uri)
-{
-	GigoloWindowPrivate *priv;
-	GigoloBookmarkList *bml;
-	GigoloBookmark *bm = NULL;
-	gboolean found = FALSE;
-	gchar *tmp_uri;
-	guint i;
-
-	g_return_val_if_fail(window != NULL, FALSE);
-	g_return_val_if_fail(uri != NULL, FALSE);
-
-	priv = GIGOLO_WINDOW_GET_PRIVATE(window);
-	bml = gigolo_settings_get_bookmarks(priv->settings);
-
-	for (i = 0; i < bml->len && ! found; i++)
-	{
-		bm = g_ptr_array_index(bml, i);
-		tmp_uri = gigolo_bookmark_get_uri_escaped(bm);
-		if (gigolo_str_equal(uri, tmp_uri))
-			found = TRUE;
-
-		g_free(tmp_uri);
-	}
-	return (found) ? bm : NULL;
-}
-
-
 void gigolo_window_mount_from_bookmark(GigoloWindow *window, GigoloBookmark *bookmark,
 									   gboolean show_dialog, gboolean show_errors)
 {
@@ -272,7 +255,11 @@ void gigolo_window_mount_from_bookmark(GigoloWindow *window, GigoloBookmark *boo
 	if (show_dialog)
 	{
 		const gchar *name = gigolo_bookmark_get_name(bookmark);
-		gchar *label = g_strdup_printf(_("Connecting to \"%s\""), (name != NULL) ? name : uri);
+		gchar *label;
+
+		if (name == NULL || gigolo_str_equal(name, GIGOLO_BOOKMARK_NAME_NONE))
+			name = uri;
+		label = g_strdup_printf(_("Connecting to \"%s\""), name);
 
 		dialog = gigolo_mount_dialog_new(GTK_WINDOW(window), label);
 		gtk_widget_show_all(dialog);
@@ -306,7 +293,7 @@ static void action_mount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *windo
 			GIGOLO_WINDOW_COL_REF, &vol, -1);
 
 		if (ref_type == GIGOLO_WINDOW_REF_TYPE_VOLUME)
-			handled = gigolo_backend_gvfs_mount_volume(priv->backend_gvfs, vol);
+			handled = gigolo_backend_gvfs_mount_volume(priv->backend_gvfs, GTK_WINDOW(window), vol);
 	}
 
 	if (! handled)
@@ -390,7 +377,7 @@ static void action_unmount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *win
 		{
 			gchar *uri;
 			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, NULL, &uri);
-			bm = gigolo_window_find_bookmark_by_uri(window, uri);
+			bm = gigolo_settings_get_bookmark_by_uri(priv->settings, uri);
 			if (bm != NULL && gigolo_bookmark_get_autoconnect(bm))
 			{	/* we don't want auto-connection to reconnect this bookmark right
 				   after we unmount it. */
@@ -398,7 +385,7 @@ static void action_unmount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *win
 			}
 			g_free(uri);
 
-			gigolo_backend_gvfs_unmount_mount(priv->backend_gvfs, mnt);
+			gigolo_backend_gvfs_unmount_mount(priv->backend_gvfs, mnt, GTK_WINDOW(window));
 		}
 	}
 }
@@ -440,12 +427,12 @@ static void action_about_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *windo
 	gtk_show_about_dialog(GTK_WINDOW(window),
 		"authors", authors,
 		"logo-icon-name", gigolo_get_application_icon_name(),
-		"comments", _("A simple frontend to easily connect to remote filesystems"),
-		"copyright", _("Copyright 2008-2009 Enrico Tröger"),
+		"comments", _("A simple frontend to easily connect/mount to local and remote filesystems"),
+		"copyright", _("Copyright 2008-2011 Enrico Tröger"),
 		"website", "http://www.uvena.de/gigolo/",
 		"version", VERSION,
 		"translator-credits", _("translator-credits"),
-		"license",  "Copyright 2008-2009 Enrico Tröger <enrico@xfce.org>\n\n"
+		"license",  "Copyright 2008-2011 Enrico Tröger <enrico@xfce.org>\n\n"
 					"This program is free software; you can redistribute it and/or modify\n"
 					"it under the terms of the GNU General Public License as published by\n"
 					"the Free Software Foundation; version 2 of the License.\n"
@@ -516,8 +503,18 @@ static void action_copy_uri_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *wi
 		if (gigolo_backend_gvfs_is_mount(mnt))
 		{
 			gchar *uri;
+			GigoloBookmark *b;
 
 			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, NULL, &uri);
+
+			b = gigolo_settings_get_bookmark_by_uri(priv->settings, uri);
+			if (b != NULL)
+			{
+				gchar *folder = gigolo_bookmark_get_folder_expanded(b);
+				setptr(uri, g_build_filename(uri, folder, NULL));
+				g_free(folder);
+			}
+
 			gtk_clipboard_set_text(gtk_clipboard_get(gdk_atom_intern("CLIPBOARD", FALSE)), uri, -1);
 
 			g_free(uri);
@@ -526,15 +523,12 @@ static void action_copy_uri_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *wi
 }
 
 
-static void action_open_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *window)
+static gpointer get_selected_mount(GigoloWindow *window)
 {
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 	GtkTreeIter iter;
 	GtkTreeModel *model = GTK_TREE_MODEL(priv->store);
 
-	if (! gigolo_settings_has_file_manager(priv->settings))
-		return;
-
 	get_selected_iter(window, &iter);
 	if (gtk_list_store_iter_is_valid(priv->store, &iter))
 	{
@@ -542,54 +536,116 @@ static void action_open_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *window
 		gtk_tree_model_get(model, &iter, GIGOLO_WINDOW_COL_REF, &mnt, -1);
 		if (gigolo_backend_gvfs_is_mount(mnt))
 		{
-#if 1
-			GError *error = NULL;
-			gchar *uri;
-			gchar *file_manager;
-			gchar *cmd;
-
-			file_manager = gigolo_settings_get_string(priv->settings, "file-manager");
-			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, NULL, &uri);
-			cmd = g_strconcat(file_manager, " ", uri, NULL);
-
-			if (! g_spawn_command_line_async(cmd, &error))
-			{
-				verbose(error->message);
-				g_error_free(error);
-			}
-
-			g_free(cmd);
-			g_free(file_manager);
-			g_free(uri);
-#else
-/*
-			GFile *file = g_mount_get_root(mnt);
-			gchar *path = g_file_get_path(file);
-
-			if (path != NULL)
-			{
-				gchar *cmd = g_strconcat("xdg-open ", path, NULL);
-				g_spawn_command_line_async(cmd, NULL);
-				g_free(cmd);
-				g_free(path);
-			}
-			else
-			{
-				// FIXME make the open command configurable or find a better solution
-				//       maybe gtk_show_uri() ?
-				GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(window),
-							GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-							_("Non-local mountpoints can't be opened."));
-				gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), _("(not yet implemented)"));
-				gtk_dialog_run(GTK_DIALOG (dialog));
-				gtk_widget_destroy(dialog);
-				verbose("Non-local mountpoints can't be opened.");
-			}
-			g_object_unref(file);
-*/
-#endif
+			return mnt;
 		}
 	}
+	return NULL;
+}
+
+
+static void action_open_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *window)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	gpointer mnt;
+
+	if (! gigolo_settings_has_file_manager(priv->settings))
+		return;
+
+	mnt = get_selected_mount(window);
+	if (mnt != NULL)
+	{
+		GError *error = NULL;
+		gchar *uri;
+		gchar *file_manager;
+		gchar *cmd;
+		GigoloBookmark *b;
+
+		file_manager = gigolo_settings_get_string(priv->settings, "file-manager");
+		gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, NULL, &uri);
+		b = gigolo_settings_get_bookmark_by_uri(priv->settings, uri);
+		if (b != NULL)
+		{
+			gchar *folder = gigolo_bookmark_get_folder_expanded(b);
+			setptr(uri, g_build_filename(uri, folder, NULL));
+			g_free(folder);
+		}
+		/* escape spaces and similar */
+		setptr(uri, g_uri_unescape_string(uri, G_URI_RESERVED_CHARS_ALLOWED_IN_USERINFO));
+
+		cmd = g_strconcat(file_manager, " \"", uri, "\"", NULL);
+		verbose("Executing open command \"%s\"", cmd);
+		if (! g_spawn_command_line_async(cmd, &error))
+		{
+			gchar *msg = g_strdup_printf(_("The command '%s' failed"), cmd);
+			gigolo_message_dialog(window, GTK_MESSAGE_ERROR, _("Error"), msg, error->message);
+			verbose("%s: %s", msg, error->message);
+			g_error_free(error);
+			g_free(msg);
+		}
+
+		g_free(cmd);
+		g_free(file_manager);
+		g_free(uri);
+	}
+}
+
+
+static void action_open_terminal_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *window)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	gpointer mnt;
+
+	if (! gigolo_settings_has_terminal(priv->settings))
+		return;
+
+	mnt = get_selected_mount(window);
+	if (mnt != NULL)
+	{
+		GError *error = NULL;
+		gchar **argv;
+		gchar *path;
+		gchar *terminal;
+
+		terminal = gigolo_settings_get_string(priv->settings, "terminal");
+		if (! g_shell_parse_argv(terminal, NULL, &argv, &error))
+		{
+			gigolo_message_dialog(window, GTK_MESSAGE_ERROR,
+				_("Error"), _("Invalid terminal command"), error->message);
+			verbose("Invalid erminal command: %s", error->message);
+			g_error_free(error);
+			return;
+		}
+
+		path = gigolo_backend_gvfs_get_mount_path(mnt);
+		if (path == NULL)
+		{
+			gchar *mount_name;
+			gchar *msg;
+			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, &mount_name, NULL);
+			msg = g_strdup_printf(_("No default location available for \"%s\""), mount_name);
+			gigolo_message_dialog(window, GTK_MESSAGE_ERROR, _("Error"), msg, NULL);
+			verbose("Mount has no default path: %s", mount_name);
+			g_free(msg);
+			g_free(mount_name);
+			g_free(terminal);
+			g_strfreev(argv);
+			return;
+		}
+
+		verbose("Executing terminal command \"%s\" in \"%s\"", terminal, path);
+		if (! g_spawn_async(path, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error))
+		{
+			gchar *msg = g_strdup_printf(_("The command '%s' failed"), terminal);
+			gigolo_message_dialog(window, GTK_MESSAGE_ERROR, _("Error"), msg, error->message);
+			verbose("%s: %s", msg, error->message);
+			g_error_free(error);
+			g_free(msg);
+		}
+
+		g_free(path);
+		g_free(terminal);
+		g_strfreev(argv);
+	}
 }
 
 
@@ -607,14 +663,15 @@ static gboolean iter_is_bookmark(GigoloWindow *window, GtkTreeModel *model, GtkT
 	gtk_tree_model_get(model, iter, GIGOLO_WINDOW_COL_REF_TYPE, &ref_type,
 									GIGOLO_WINDOW_COL_REF, &ref, -1);
 
-	if (ref_type == GIGOLO_WINDOW_REF_TYPE_MOUNT)
+	if (ref_type == GIGOLO_WINDOW_REF_TYPE_MOUNT && gigolo_backend_gvfs_is_mount(ref))
 	{
 		gchar *uri;
 		gboolean found = FALSE;
+		GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 
 		gigolo_backend_gvfs_get_name_and_uri_from_mount(ref, NULL, &uri);
 
-		found = (gigolo_window_find_bookmark_by_uri(window, uri) != NULL);
+		found = (gigolo_settings_get_bookmark_by_uri(priv->settings, uri) != NULL);
 
 		g_free(uri);
 		return found;
@@ -624,21 +681,52 @@ static gboolean iter_is_bookmark(GigoloWindow *window, GtkTreeModel *model, GtkT
 }
 
 
+static gboolean iter_is_mount(GtkTreeModel *model, GtkTreeIter *iter)
+{
+	gint ref_type;
+	gpointer ref;
+
+	gtk_tree_model_get(model, iter, GIGOLO_WINDOW_COL_REF_TYPE, &ref_type,
+									GIGOLO_WINDOW_COL_REF, &ref, -1);
+
+	if (ref_type == GIGOLO_WINDOW_REF_TYPE_MOUNT)
+	{
+		return gigolo_backend_gvfs_is_mount(ref);
+	}
+
+	return FALSE;
+}
+
+
+static void update_create_edit_bookmark_action_label(GtkAction *action, gboolean is_bookmark)
+{
+	gtk_action_set_sensitive(action, TRUE);
+	if (is_bookmark)
+		gtk_action_set_label(action, _("Edit _Bookmark"));
+	else
+		gtk_action_set_label(action, _("Create _Bookmark"));
+}
+
+
 static void update_sensitive_buttons(GigoloWindow *window, GtkTreeModel *model, GtkTreeIter *iter)
 {
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 	gint ref_type;
-	gboolean is_bookmark = FALSE;
+	gboolean is_bookmark,is_mount, open_possible, open_terminal_possible;
 
 	if (iter != NULL && gtk_list_store_iter_is_valid(priv->store, iter))
 	{
 		gtk_tree_model_get(model, iter, GIGOLO_WINDOW_COL_REF_TYPE, &ref_type, -1);
 		is_bookmark = iter_is_bookmark(window, model, iter);
+		is_mount = iter_is_mount(model, iter);
+		open_possible = is_mount && gigolo_settings_has_file_manager(priv->settings);
+		open_terminal_possible = is_mount && gigolo_settings_has_terminal(priv->settings);
 
-		gtk_action_set_sensitive(priv->action_connect, (ref_type != GIGOLO_WINDOW_REF_TYPE_MOUNT));
+		/* gtk_action_set_sensitive(priv->action_connect, (ref_type != GIGOLO_WINDOW_REF_TYPE_MOUNT));*/
 		gtk_action_set_sensitive(priv->action_disconnect, (ref_type == GIGOLO_WINDOW_REF_TYPE_MOUNT));
-		gtk_action_set_sensitive(priv->action_bookmark_create, ! is_bookmark);
-		gtk_action_set_sensitive(priv->action_open, gigolo_settings_has_file_manager(priv->settings));
+		update_create_edit_bookmark_action_label(priv->action_bookmark_create, is_bookmark);
+		gtk_action_set_sensitive(priv->action_open, open_possible);
+		gtk_action_set_sensitive(priv->action_open_terminal, open_terminal_possible);
 		gtk_action_set_sensitive(priv->action_copyuri, (ref_type == GIGOLO_WINDOW_REF_TYPE_MOUNT));
 	}
 	else
@@ -647,6 +735,7 @@ static void update_sensitive_buttons(GigoloWindow *window, GtkTreeModel *model,
 		gtk_action_set_sensitive(priv->action_disconnect, FALSE);
 		gtk_action_set_sensitive(priv->action_bookmark_create, FALSE);
 		gtk_action_set_sensitive(priv->action_open, FALSE);
+		gtk_action_set_sensitive(priv->action_open_terminal, FALSE);
 		gtk_action_set_sensitive(priv->action_copyuri, FALSE);
 	}
 }
@@ -705,7 +794,7 @@ static void mounts_changed_cb(G_GNUC_UNUSED GigoloBackendGVFS *backend, GigoloWi
 static void mount_operation_failed_cb(G_GNUC_UNUSED GigoloBackendGVFS *backend, const gchar *message,
 								   const gchar *error_message, GigoloWindow *window)
 {
-	gigolo_message_dialog((gpointer) window, GTK_MESSAGE_ERROR, _("Error"), message, error_message);
+	gigolo_message_dialog(window, GTK_MESSAGE_ERROR, _("Error"), message, error_message);
 }
 
 
@@ -827,6 +916,12 @@ void gigolo_window_update_bookmarks(GigoloWindow *window)
 
 	/* writing to the 'settings' property will update the menus */
 	g_object_set(priv->action_bookmarks, "settings", priv->settings, NULL);
+	g_object_set(priv->bookmark_panel, "settings", priv->settings, NULL);
+
+	/* update the popup menu items */
+	tree_selection_changed_cb(NULL, window);
+
+	gigolo_backend_gvfs_update_mounts_and_volumes(priv->backend_gvfs);
 }
 
 
@@ -885,16 +980,17 @@ static void action_create_bookmark_cb(G_GNUC_UNUSED GtkAction *button, GigoloWin
 		if (gigolo_backend_gvfs_is_mount(mnt))
 		{
 			gchar *uri, *name;
+			GigoloBookmark *bm;
+			GtkWidget *edit_dialog;
 
 			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, &name, &uri);
 
-			if (gigolo_window_find_bookmark_by_uri(window, uri) == NULL)
+			bm = gigolo_settings_get_bookmark_by_uri(priv->settings, uri);
+			if (bm == NULL)
 			{
-				GigoloBookmark *bm = gigolo_bookmark_new_from_uri(name, uri);
+				bm = gigolo_bookmark_new_from_uri(name, uri);
 				if (gigolo_bookmark_is_valid(bm))
 				{
-					GtkWidget *edit_dialog;
-
 					/* show the bookmark edit dialog and add the bookmark only if it was
 					 * not cancelled */
 					edit_dialog = gigolo_bookmark_edit_dialog_new_with_bookmark(
@@ -915,8 +1011,21 @@ static void action_create_bookmark_cb(G_GNUC_UNUSED GtkAction *button, GigoloWin
 				g_object_unref(bm);
 			}
 			else
-				verbose("Bookmark for %s already exists", uri);
+			{
+				/* bookmark exists */
+				edit_dialog = gigolo_bookmark_edit_dialog_new_with_bookmark(
+					window, GIGOLO_BE_MODE_EDIT, bm);
+				if (gigolo_bookmark_edit_dialog_run(
+						GIGOLO_BOOKMARK_EDIT_DIALOG(edit_dialog)) == GTK_RESPONSE_OK)
+				{
+					/* this fills the values of the dialog into 'bm' */
+					g_object_set(edit_dialog, "bookmark-update", bm, NULL);
 
+					gigolo_window_update_bookmarks(window);
+					gigolo_settings_write(priv->settings, GIGOLO_SETTINGS_BOOKMARKS);
+				}
+				gtk_widget_destroy(edit_dialog);
+			}
 			g_free(uri);
 			g_free(name);
 		}
@@ -928,7 +1037,7 @@ static void gigolo_window_show_side_panel(GigoloWindow *window, gboolean show)
 {
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 
-	if (show && gigolo_backend_gvfs_is_scheme_supported("smb"))
+	if (show)
 		gtk_widget_show(priv->hbox_pane);
 	else
 		gtk_widget_hide(priv->hbox_pane);
@@ -1027,15 +1136,6 @@ static void view_mode_action_set_active(GigoloWindow *window, gint val)
 }
 
 
-static void action_set_sensitive(GigoloWindow *window, const gchar *name, gboolean set)
-{
-	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
-	GtkAction *action = gtk_action_group_get_action(priv->action_group, name);
-
-	gtk_action_set_sensitive(action, set);
-}
-
-
 static void gigolo_window_systray_notify_cb(GtkStatusIcon *sicon, GParamSpec *pspec, GtkWindow *window)
 {
 	const gchar *name;
@@ -1109,8 +1209,6 @@ static void gigolo_window_settings_notify_cb(GigoloSettings *settings, GParamSpe
 
 static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 {
-	GError *error = NULL;
-	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 	const gchar *ui_markup =
 	"<ui>"
 		"<menubar>"
@@ -1136,6 +1234,7 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 				"<menuitem action='Bookmarks'/>"
 				"<separator/>"
 				"<menuitem action='Open'/>"
+				"<menuitem action='OpenTerminal'/>"
 				"<menuitem action='CopyURI'/>"
 			"</menu>"
 			"<menu action='Help'>"
@@ -1158,6 +1257,7 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 
 		"<popup name='treemenu'>"
 			"<menuitem action='Open'/>"
+			"<menuitem action='OpenTerminal'/>"
 			"<menuitem action='CopyURI'/>"
 			"<menuitem action='CreateBookmark'/>"
 			"<separator/>"
@@ -1172,6 +1272,7 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 			"<toolitem action='EditBookmarks'/>"
 			"<separator/>"
 			"<toolitem action='Open'/>"
+			"<toolitem action='OpenTerminal'/>"
 			"<separator/>"
 			"<toolitem action='Quit'/>"
 		"</toolbar>"
@@ -1195,6 +1296,8 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 			N_("Disconnect the selected resource"), G_CALLBACK(action_unmount_cb) },
 		{ "Open", GTK_STOCK_OPEN, NULL, "<Ctrl>o",
 			N_("Open the selected resource with a file manager"), G_CALLBACK(action_open_cb) },
+		{ "OpenTerminal", NULL, _("Open in _Terminal"), "<Ctrl>t",
+			N_("Start a terminal from here"), G_CALLBACK(action_open_terminal_cb) },
 		{ "CopyURI", GTK_STOCK_COPY, N_("Copy _URI"), "<Ctrl>c", NULL, G_CALLBACK(action_copy_uri_cb) },
 		{ "Quit", GTK_STOCK_QUIT, NULL, "<Ctrl>q", N_("Quit Gigolo"), G_CALLBACK(action_quit_cb) },
 		{ "OnlineHelp", GTK_STOCK_HELP, _("Online Help"), NULL, NULL, G_CALLBACK(action_help_cb) },
@@ -1218,7 +1321,10 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 		{ "ViewSymbols", NULL, N_("View as _Detailed List"), NULL, NULL, 1 },
 	};
 	const guint radio_entries_n = G_N_ELEMENTS(radio_entries);
-
+	GError *error = NULL;
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	GtkAction *open_terminal;
+	GtkIconTheme *icon_theme;
 
 	priv->action_bookmarks = gigolo_menu_button_action_new(
 		"Bookmarks", _("_Bookmarks"), _("Choose a bookmark to connect to"),
@@ -1237,6 +1343,14 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 	gtk_ui_manager_insert_action_group(ui_manager, priv->action_group, 0);
 	gtk_window_add_accel_group(GTK_WINDOW(window), gtk_ui_manager_get_accel_group(ui_manager));
 
+	/* set terminal icon for the "OpenTerminal" action */
+	open_terminal = gtk_action_group_get_action(priv->action_group, "OpenTerminal");
+	icon_theme = gtk_icon_theme_get_for_screen(gtk_widget_get_screen(GTK_WIDGET(window)));
+	if (gtk_icon_theme_has_icon(icon_theme, "utilities-terminal"))
+		gtk_action_set_icon_name(open_terminal, "utilities-terminal");
+	else
+		gtk_action_set_stock_id(open_terminal, GTK_STOCK_OPEN);
+
 	if (! gtk_ui_manager_add_ui_from_string(ui_manager, ui_markup, -1, &error))
 	{
 		verbose("User interface couldn't be created: %s", error->message);
@@ -1289,7 +1403,7 @@ static void create_tree_view(GigoloWindow *window)
 		column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
 			"gicon", GIGOLO_WINDOW_COL_PIXBUF, NULL);
 		gtk_tree_view_column_set_sort_indicator(column, FALSE);
-		gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
+		gtk_tree_view_column_set_resizable(column, TRUE);
 		gtk_tree_view_append_column(GTK_TREE_VIEW(priv->treeview), column);
 	}
 
@@ -1298,7 +1412,7 @@ static void create_tree_view(GigoloWindow *window)
 		_("Connected"), renderer, "active", GIGOLO_WINDOW_COL_IS_MOUNTED, NULL);
 	gtk_tree_view_column_set_sort_indicator(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, GIGOLO_WINDOW_COL_IS_MOUNTED);
-	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(priv->treeview), column);
 	g_signal_connect(renderer, "toggled", G_CALLBACK(tree_mounted_col_toggled_cb), window);
 
@@ -1307,7 +1421,7 @@ static void create_tree_view(GigoloWindow *window)
 		_("Service Type"), renderer, "text", GIGOLO_WINDOW_COL_SCHEME, NULL);
 	gtk_tree_view_column_set_sort_indicator(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, GIGOLO_WINDOW_COL_SCHEME);
-	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(priv->treeview), column);
 
 	renderer = gtk_cell_renderer_text_new();
@@ -1315,7 +1429,7 @@ static void create_tree_view(GigoloWindow *window)
 		_("Name"), renderer, "text", GIGOLO_WINDOW_COL_NAME, NULL);
 	gtk_tree_view_column_set_sort_indicator(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, GIGOLO_WINDOW_COL_NAME);
-	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(priv->treeview), column);
 
 	gtk_tree_view_set_model(GTK_TREE_VIEW(priv->treeview), GTK_TREE_MODEL(priv->store));
@@ -1371,6 +1485,66 @@ static void create_icon_view(GigoloWindow *window)
 }
 
 
+static GtkWidget *gigolo_window_create_panel(GigoloWindow *window)
+{
+	GtkWidget *panel_pane, *label;
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+
+	panel_pane = gtk_hpaned_new();
+	gtk_paned_set_position(GTK_PANED(panel_pane), 200);
+
+	priv->notebook_panel = gtk_notebook_new();
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(priv->notebook_panel), GTK_POS_LEFT);
+	gtk_widget_show(priv->notebook_panel);
+
+	label = gtk_label_new(_("Bookmarks"));
+	gtk_label_set_angle(GTK_LABEL(label), 90.0);
+	gtk_widget_show(label);
+
+	priv->bookmark_panel = gigolo_bookmark_panel_new(window);
+	gtk_widget_show(priv->bookmark_panel);
+	gtk_notebook_append_page(GTK_NOTEBOOK(priv->notebook_panel), priv->bookmark_panel, label);
+
+	label = gtk_label_new(_("Network"));
+	gtk_label_set_angle(GTK_LABEL(label), 90.0);
+	gtk_widget_show(label);
+
+	priv->browse_panel = gigolo_browse_network_panel_new(window);
+	gtk_widget_show(priv->browse_panel);
+	gtk_notebook_append_page(GTK_NOTEBOOK(priv->notebook_panel), priv->browse_panel, label);
+
+	priv->hbox_view = gtk_hbox_new(FALSE, 0);
+	priv->hbox_pane = gtk_hbox_new(FALSE, 0);
+
+	gtk_paned_add1(GTK_PANED(panel_pane), priv->hbox_pane);
+	gtk_paned_add2(GTK_PANED(panel_pane), priv->hbox_view);
+	gtk_box_pack_start(GTK_BOX(priv->hbox_pane), priv->notebook_panel, TRUE, TRUE, 0);
+
+	return panel_pane;
+}
+
+
+static void update_side_panel(GigoloWindow *window)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	guint panel_position;
+
+	if (! gigolo_backend_gvfs_is_scheme_supported("smb"))
+		gtk_widget_destroy(priv->browse_panel);
+
+	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(priv->notebook_panel)) < 2)
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(priv->notebook_panel), FALSE);
+
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(priv->notebook_panel),
+		gigolo_settings_get_integer(priv->settings, "last-panel-page"));
+
+	panel_position = gigolo_settings_get_integer(priv->settings, "panel-position");
+	if (panel_position <= 0)
+		panel_position = 200;
+	gtk_paned_set_position(GTK_PANED(priv->panel_pane), panel_position);
+}
+
+
 static void gigolo_window_init(GigoloWindow *window)
 {
 	GtkWidget *menubar, *panel_pane;
@@ -1404,7 +1578,7 @@ static void gigolo_window_init(GigoloWindow *window)
 	gtk_container_add(GTK_CONTAINER(priv->swin_iconview), priv->iconview);
 
 	/* Init the GVfs backend */
-	priv->backend_gvfs = gigolo_backend_gvfs_new(priv->store);
+	priv->backend_gvfs = gigolo_backend_gvfs_new();
 	g_signal_connect(priv->backend_gvfs, "mounts-changed", G_CALLBACK(mounts_changed_cb), window);
 	g_signal_connect(priv->backend_gvfs, "operation-failed",
 		G_CALLBACK(mount_operation_failed_cb), window);
@@ -1428,6 +1602,7 @@ static void gigolo_window_init(GigoloWindow *window)
 	priv->action_disconnect = gtk_action_group_get_action(priv->action_group, "Disconnect");
 	priv->action_bookmark_create = gtk_action_group_get_action(priv->action_group, "CreateBookmark");
 	priv->action_open = gtk_action_group_get_action(priv->action_group, "Open");
+	priv->action_open_terminal = gtk_action_group_get_action(priv->action_group, "OpenTerminal");
 	priv->action_copyuri = gtk_action_group_get_action(priv->action_group, "CopyURI");
 
 	g_object_set(priv->action_bookmarks, "is-important", TRUE, NULL);
@@ -1435,20 +1610,10 @@ static void gigolo_window_init(GigoloWindow *window)
 		"is-important", TRUE, NULL);
 
 	/* Panel */
-	panel_pane = gtk_hpaned_new();
-	gtk_paned_set_position(GTK_PANED(panel_pane), 200);
-
-	priv->browse_panel = gigolo_browse_network_panel_new(window);
-	gtk_widget_show(priv->browse_panel);
+	priv->panel_pane = panel_pane = gigolo_window_create_panel(window);
 
 	/* Pack the widgets altogether */
 	priv->vbox = gtk_vbox_new(FALSE, 0);
-	priv->hbox_view = gtk_hbox_new(FALSE, 0);
-	priv->hbox_pane = gtk_hbox_new(FALSE, 0);
-
-	gtk_paned_add1(GTK_PANED(panel_pane), priv->hbox_pane);
-	gtk_paned_add2(GTK_PANED(panel_pane), priv->hbox_view);
-	gtk_box_pack_start(GTK_BOX(priv->hbox_pane), priv->browse_panel, TRUE, TRUE, 0);
 
 	gtk_box_pack_start(GTK_BOX(priv->vbox), menubar, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(priv->vbox), priv->toolbar, FALSE, FALSE, 0);
@@ -1488,6 +1653,8 @@ GtkWidget *gigolo_window_new(GigoloSettings *settings)
 
 	g_object_set(priv->action_bookmarks, "settings", settings, NULL);
 
+	g_object_set(priv->backend_gvfs, "parent", window, "store", priv->store, NULL);
+
 	gigolo_window_set_toolbar_style(GIGOLO_WINDOW(window),
 		gigolo_settings_get_integer(settings, "toolbar-style"));
 	gigolo_window_set_toolbar_orientation(GIGOLO_WINDOW(window),
@@ -1496,7 +1663,7 @@ GtkWidget *gigolo_window_new(GigoloSettings *settings)
 	state = gigolo_settings_get_boolean(settings, "show-panel");
 	gigolo_window_show_side_panel(GIGOLO_WINDOW(window), state);
 	toggle_action_set_active(GIGOLO_WINDOW(window), "ShowPanel", state);
-	action_set_sensitive(GIGOLO_WINDOW(window), "ShowPanel", gigolo_backend_gvfs_is_scheme_supported("smb"));
+	update_side_panel(GIGOLO_WINDOW(window));
 	/* Show Toolbar */
 	state = gigolo_settings_get_boolean(settings, "show-toolbar");
 	gigolo_window_show_toolbar(GIGOLO_WINDOW(window), state);
@@ -1542,3 +1709,15 @@ GigoloSettings *gigolo_window_get_settings(GigoloWindow *window)
 }
 
 
+GigoloBackendGVFS *gigolo_window_get_backend(GigoloWindow *window)
+{
+	GigoloWindowPrivate *priv;
+
+	g_return_val_if_fail(window != NULL, NULL);
+
+	priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+
+	return priv->backend_gvfs;
+}
+
+
